import { HttpException, HttpStatus, Injectable } from "@nestjs/common";
import { Constants } from "src/utils/constants";
import { CreateBlogPostDto } from "./dto/create-blog-post.dto";
import { UpdateBlogPostDto } from "./dto/update-blog-post.dto";
import { BlogPost } from "./entities/blog-post.entity";
import { BlogPostRepository } from "./blog-post.repository";
import { ResponseUtils } from "src/utils/response.utils";
import { BlogPostTagsRepository } from "../blog-post-tags/blog-post-tags.repository";
import { BlogTagRepository } from "../blog-tag/blog-tag.repository";
import { CreateBlogPostTagsDto } from "../blog-post-tags/dto/create-blog-post-tags.dto";
import { BlogRelatedPostsRepository } from "../blog-related-posts/blog-related-posts.repository";
import { CreateBlogRelatedPostsDto } from "../blog-related-posts/dto/create-blog-related-posts.dto";
import { UploadedMulterFileI } from "src/modules/SpacesModule/SpacesService";
import { DoSpacesService } from "src/modules/SpacesModule/SpacesService/doSpacesService";
import { urlSlug } from "src/helper/common-helper";

@Injectable()
export class BlogPostService {
    constructor(
      private readonly repository: BlogPostRepository,
      private readonly blogTagRepository: BlogTagRepository,
      private readonly blogPostTagsRepository: BlogPostTagsRepository,
      private readonly blogRelatedPostsRepository: BlogRelatedPostsRepository,
      private readonly doSpaceService: DoSpacesService
    ) {
    }

    async create(
      dto: CreateBlogPostDto,
      files: {
          photo?: UploadedMulterFileI
      }
    ): Promise<any | Error> {
        if (files && files.photo) {
            const photo: any = await this.doSpaceService.uploadFile(files.photo[0], "Article/");
            dto.photo = photo;
        } else dto.photo = null;

        dto.urlSlug = urlSlug(dto.urlSlug);

        const data = await this.repository.createEntity(dto);
        if (!data) {
            throw new HttpException(
              Constants.CREATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        let postTags: any;
        if (dto.tags.length > 0) {
            postTags = JSON.parse(dto.tags);
            for (const element of postTags) {
                let tagData = await this.blogTagRepository.findOne({
                    where: { tagName: element }
                });
                if (!tagData) {
                    throw new HttpException(
                      Constants.NOT_FOUND,
                      HttpStatus.NOT_FOUND
                    );
                }
                let postTagsDto = {
                    "postId": data.id,
                    "postTitle": dto.title,
                    "tagId": tagData.id,
                    "tagName": tagData.tagName
                };
                const res = await this.blogPostTagsRepository.createBlogPostTags(postTagsDto as CreateBlogPostTagsDto);
                if (!res) {
                    throw new HttpException(
                      Constants.CREATE_FAILED,
                      HttpStatus.BAD_REQUEST
                    );
                }
            }
        }
        let relatedPosts: any;
        if (dto.relatedPosts.length > 0) {
            relatedPosts = JSON.parse(dto.relatedPosts);
            for (const element of relatedPosts) {
                let postData = await this.repository.findOne({ where: { title: element } });
                if (!postData) {
                    throw new HttpException(
                      Constants.NOT_FOUND,
                      HttpStatus.NOT_FOUND
                    );
                }
                let relatedPostsDto = {
                    "postId": data.id,
                    "postTitle": dto.title,
                    "relatedPostId": postData.id,
                    "relatedPostTitle": postData.title
                };
                const res = await this.blogRelatedPostsRepository.createBlogRelatedPosts(relatedPostsDto as CreateBlogRelatedPostsDto);
                if (!res) {
                    throw new HttpException(
                      Constants.CREATE_FAILED,
                      HttpStatus.BAD_REQUEST
                    );
                }
            }
        }
        return ResponseUtils.successResponseHandler(201, "Blog post created successfully", "data", data);
    }

    async findAll(): Promise<BlogPost[] | Error> {
        const data = await this.repository.findAll();
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        return ResponseUtils.successResponseHandler(200, "Blog post list fetched successfully", "data", data);
    }

    async filterPressRelease(
      page: number,
      limit: number,
      category?: string
    ): Promise<any | Error> {
        let count: any;
        let data: any;
        if (category) {
            count = await this.repository.find({
                where: {
                    categoryName: category,
                    isDeleted: false
                }
            });
            data = await this.repository.find({
                where: {
                    categoryName: category,
                    isDeleted: false
                },
                take: limit,
                skip: limit * (page - 1)
            });
        } else {
            count = await this.repository.find({
                where: {
                    isDeleted: false
                }
            });
            data = await this.repository.find({
                where: {
                    isDeleted: false
                },
                take: limit,
                skip: limit * (page - 1)
            });
        }

        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        const pageCount = Math.ceil(count.length / limit);
        return {
            page: page,
            limit: limit,
            pageCount: pageCount,
            data
        };
    }

    async findOne(id: string): Promise<any | Error> {
        let data: any = await this.repository.findOneEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.NOT_FOUND
            );
        }
        if (data) {
            let viewCount: number;
            let totalViews = data[0].totalViews;
            let postId = data[0].id;
            viewCount = ((totalViews ? totalViews : 0) - 0) + (1);
            let blogPostDto = {
                "totalViews": viewCount
            };
            await this.repository.updateBlogViewCount(postId, blogPostDto as UpdateBlogPostDto);
        }
        const tags = await this.blogPostTagsRepository.findBlogPostTagsByPostId(id);
        const relatedPosts = await this.blogRelatedPostsRepository.findBlogRelatedPostsByPostId(id);
        let relatedPostsDetails: any = [];
        if (relatedPosts) {
            for (const index in relatedPosts) {
                let postId = `${relatedPosts[index].relatedPostId}`;
                let postData = await this.repository.findOneEntity(postId);
                relatedPostsDetails.push(postData);
            }
        }
        data[0]["tags"] = tags;
        data[0]["relatedPosts"] = relatedPosts;
        data[0]["relatedPostsDetails"] = relatedPostsDetails;
        return ResponseUtils.successResponseHandler(200, "Blog post fetched successfully", "data", data);
    }

    async update(
      id: string,
      dto: UpdateBlogPostDto,
      files: {
          photo?: UploadedMulterFileI
      }
    ): Promise<BlogPost | Error> {
        if (files && files.photo) {
            const photo: any = await this.doSpaceService.uploadFile(files.photo[0], "Article/");
            dto.photo = photo;
        }
        await this.blogPostTagsRepository.deleteBlogPostTagsByBlogID(id);
        let postTags: any;
        if (dto.tags.length > 0) {
            postTags = JSON.parse(dto.tags);
            for (const element of postTags) {
                let tagData = await this.blogTagRepository.findOne({ where: { tagName: element } });
                if (!tagData) {
                    throw new HttpException(
                      Constants.NOT_FOUND,
                      HttpStatus.NOT_FOUND
                    );
                }
                let postTagsDto = {
                    "postId": id,
                    "postTitle": dto.title,
                    "tagId": tagData.id,
                    "tagName": tagData.tagName
                };
                await this.blogPostTagsRepository.createBlogPostTags(postTagsDto as CreateBlogPostTagsDto);
            }
        }
        await this.blogRelatedPostsRepository.deleteBlogRelatedPostsByBlogID(id);
        let relatedPosts: any;
        if (dto.relatedPosts.length > 0) {
            relatedPosts = JSON.parse(dto.relatedPosts);
            for (const element of relatedPosts) {
                let postData = await this.repository.findOne({
                    where: { title: element }
                });
                if (!postData) {
                    throw new HttpException(
                      Constants.NOT_FOUND,
                      HttpStatus.NOT_FOUND
                    );
                }
                let relatedPostsDto = {
                    "postId": id,
                    "postTitle": dto.title,
                    "relatedPostId": postData.id,
                    "relatedPostTitle": postData.title
                };
                await this.blogRelatedPostsRepository.createBlogRelatedPosts(relatedPostsDto as CreateBlogRelatedPostsDto);
            }
        }
        dto.urlSlug = urlSlug(dto.urlSlug);
        const data = await this.repository.updateBlogPost(id, dto);
        if (!data) {
            throw new HttpException(
              Constants.UPDATE_FAILED,
              HttpStatus.BAD_REQUEST
            );
        }
        return ResponseUtils.successResponseHandler(200, "Blog post updated successfully", "data", data);
    }

    async remove(id: string): Promise<boolean | Error> {
        const data = await this.repository.deleteEntity(id);
        if (!data) {
            throw new HttpException(
              Constants.NOT_FOUND,
              HttpStatus.BAD_REQUEST
            );
        }
        await this.blogPostTagsRepository.deleteBlogPostTagsByBlogID(id);
        await this.blogRelatedPostsRepository.deleteBlogRelatedPostsByBlogID(id);
        return ResponseUtils.buildDeletedData(data, 200, "Blog post deleted successfully");
    }
}
